--- Configs
-- @author Enno Sylvester
-- @copyright 2017
-- @module eatglobal.configs

local configs = {
	version = 1.3,
}

local function init ()
	if type(game) == "table" then
		if type(game.eat1963) ~= "table" then
			game.eat1963 = {}
		end
		if type(game.eat1963.configs) ~= "table" then
			game.eat1963.configs = {}
		end
		return true
	else
		return false
	end
end

function configs.createConfig ( id, params )
	local function checkLogFn (id)
		local config = game.eat1963.configs[id]
		if config.env.debug then
			config.logFn = configs.eatglobal.utils.createLogFn (id)
		else
			config.logFn = function (...) end
		end
	end
	local function createSetDebugFn ( id )
		local id = id
		local function setFn ( _debug )
			game.eat1963.configs[id].env.debug = _debug
			checkLogFn (id)
		end
		return setFn
	end
	local function createGetIdFn (id)
		local id = id
		local function getFn ()
			return id
		end
		return getFn
	end
	
	local function createSaveModSettingsFn (id)
		local id = id
		local function setFn ()
			configs.eatglobal.utils.saveData(game.eat1963.configs[id].modSettings, game.eat1963.configs[id].env.mod_settingsFile)
		end
		return setFn
	end
	
	local function createSaveUserConfigFn (id)
		local id = id
		local function setFn ()
			configs.eatglobal.userSettings.flushSettings (id)
		end
		return setFn
	end
	
	local function createImportUserConfigFn (id)
		local id = id
		local function getFn (userConfig)
			game.eat1963.configs[id].userConfig = 
				configs.eatglobal.utils.proxyTable(configs.eatglobal.userSettings.importUserSettings(id, userConfig))
		end
		return getFn
	end
	
	local function createInitializeUserConfigFn (id)
		local id = id
		local function getFn (defaultUserSettings)
			game.eat1963.configs[id].userConfig = 
				configs.eatglobal.utils.proxyTable(configs.eatglobal.userSettings.create(id, defaultUserSettings))
		end
		return getFn
	end
	
	if (init () and (type(id) == "string")) then
		if not game.eat1963.configs[id] then
			local temp = params or {}
			if (temp.env == nil) then
				temp.env = {}
			end
			if (temp.userConfig == nil) then
				temp.userConfig = {}
			end
			if (temp.modSettings == nil) then
				temp.modSettings = {}
			end
			if (temp.modifierSequenceList == nil) then
				temp.modifierSequenceList = {
					["01"] = id,
				}
			end
			-- Umgebung (env(ironment))
			temp.env.modPath = configs.eatglobal.utils.getModPath ()
			temp.env.settingsFile = temp.env.modPath.."settings.lua"
			temp.env.mod_settingsFile = temp.env.modPath.."mod_settings.lua"
			temp.env.scriptsPath = temp.env.modPath.."res/scripts/"
			temp.env.isWinOS = (os.getenv("OS") == "Windows_NT")
			
			temp.setDebug = createSetDebugFn (id)
			temp.getId = createGetIdFn (id)
			-- Mod-Settings
			temp.modSettings = configs.eatglobal.utils.readData(temp.env.mod_settingsFile)
			temp.saveModSettings = createSaveModSettingsFn (id)
			-- User-Settings
			temp.initializeUserConfig = createInitializeUserConfigFn (id)
			temp.saveUserConfig = createSaveUserConfigFn (id)
			temp.importUserConfig = createImportUserConfigFn (id)
			-- zuweisen der Daten
			game.eat1963.configs[id] = temp
			
			--
			checkLogFn(id)
		end
		return game.eat1963.configs[id]
	else
		return {}
	end
end

function configs.getConfig ( id )
	if init () and (type(game.eat1963.configs[id]) == "table") then
		return game.eat1963.configs[id]
	else
		return {}
	end
end

function configs.existsConfig ( id )
	return (init () and (type(game.eat1963.configs[id]) == "table"))
end

return configs