--- Usersettings
-- @author Enno Sylvester
-- @copyright 2017
-- @module eatglobal.userSettings

require "stringutil"

local userSettings = {
	version = 1.2,
}

local function init ()
	if type(game) == "table" then
		if type(game.eat1963) ~= "table" then
			game.eat1963 = {}
		end
		if type(game.eat1963.userSettings) ~= "table" then
			game.eat1963.userSettings = {}
		end
		return true
	else
		return false
	end
end

local function doImportSettings(modSettings, importSettings)
	modSettings.values = {}
	for key, value in pairs(modSettings.defaultSettings) do
		if (value.type ~= nil) then
			local importValue = importSettings[key]
			if (type(importValue) == value.type) then
				if value.type == "number" then
					if (value.min ~= nil) then
						importValue = math.max(value.min, importValue)
					end
					if (value.max ~= nil) then
						importValue = math.min(value.max, importValue)
					end
				end
				modSettings.values[key] = importValue
			else
				if (value.default ~= nil) then
					modSettings.values[key] = value.default
				else
					if (value.type == "boolean") then
						modSettings.values[key] = false
					elseif (value.type == "number") then
						modSettings.values[key] = 0
					elseif (value.type == "string") then
						modSettings.values[key] = ""
					end
				end
			end
		end
	end
	
	return modSettings
end

function userSettings.create (id, defaultSettings)
	if init() and (type(defaultSettings) == "table") then
		local modSettings = {
			defaultSettings = defaultSettings,
			modPath = userSettings.eatglobal.utils.getModPath(),
			values = {},
		}
		local fileSettings = userSettings.eatglobal.utils.readData(modSettings.modPath.."settings.lua")
		if (type(fileSettings) ~= "table") then
			fileSettings = {}
		end
		
		game.eat1963.userSettings[id] = doImportSettings(modSettings, fileSettings)
		
		if not userSettings.eatglobal.utils.fileExists(modSettings.modPath.."settings.lua") then
			userSettings.flushSettings(id)
		end
		
		return game.eat1963.userSettings[id].values
	else
		return {}
	end
end

function userSettings.importUserSettings (id, userSettings)
	if init () and (type(game.eat1963.userSettings[id]) == "table") and (type(userSettings) == "table") then
		game.eat1963.userSettings[id] = doImportSettings(game.eat1963.userSettings[id], userSettings)
		return game.eat1963.userSettings[id].values
	else
		return {}
	end
end

function userSettings.getSettings (id)
	if init () and (type(game.eat1963.userSettings[id]) == "table") then
		return game.eat1963.userSettings[id].values
	else
		return {}
	end
end

function userSettings.existsSettings (id)
	return (init () and (type(game.eat1963.userSettings[id]) == "table"))
end

function userSettings.flushSettings (id)
	if init () and (type(game.eat1963.userSettings[id]) == "table") then
		userSettings.eatglobal.utils.saveData(game.eat1963.userSettings[id].values, game.eat1963.userSettings[id].modPath.."settings.lua")
	end
end

return userSettings
